# Copyright (c) 2005-2006, Samuel Schluep, ETH Zurich and Infrae
# All rights reserved. See also LICENSE.txt

# Zope
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
# Silva
from Products.Silva import mangle
# Silva Document
from Products.SilvaDocument import mixedcontentsupport
# Silva Glossary
from Products.SilvaGlossary import install
from Products.SilvaGlossary.helpers import glossaryLinkHelper


class GlossaryParagraphSupport(mixedcontentsupport.ParagraphSupport):
    """Mixed content support with glossary link handling.

    This class extends the default Silva Document handling of paragraph
    markup to include glossary links. If the link point to a glossary entry
    and the Silva Glossary product is installed the glossary link rendering
    comes into effect.
    """
    security = ClassSecurityInfo()

    def _renderHtmlHelper(self, node, view_type):
        # This method has been copied from Silva Document and
        # the link rendering has been adapted to include glossary links.
        result = []
        for child in node.childNodes:
            if child.nodeType == child.TEXT_NODE:
                result.append(mangle.entities(child.nodeValue))
                continue
            if child.nodeType != child.ELEMENT_NODE:
                continue
            if child.nodeName == 'strong':
                result.append('<strong>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</strong>')
            elif child.nodeName == 'em':
                result.append('<em>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</em>')
            elif child.nodeName == 'super':
                result.append('<sup>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</sup>')
            elif child.nodeName == 'sub':
                result.append('<sub>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</sub>')
            elif child.nodeName == 'abbr':
                result.append('<abbr')
                if child.getAttribute('title'):
                    result.append(
                        ' title="%s"' % child.getAttribute('title'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</abbr>')
            elif child.nodeName == 'acronym':
                result.append('<acronym')
                if child.getAttribute('title'):
                    result.append(
                        ' title="%s"' % child.getAttribute('title'))
                result.append('>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</acronym>')
            elif child.nodeName == 'link':
                path = child.getAttribute('url')
                ### Silva Glossary: process glossary link if target is a
                ### Silva Glossary Entry and SilvaGlossary is installed.
                url, obj = glossaryLinkHelper(node, path)
                if obj != None and \
                   obj.meta_type == 'Silva Glossary Entry' and \
                   install.is_installed(child.get_root()):
                    result.append(renderGlossaryLink(self, obj, child,
                                  view_type))
                ### Normal Silva Document behavior
                else:
                #    url = self._linkHelper(node, path)
                    result.append('<a href="%s"' %  mangle.entities(url))
                    if child.hasAttribute('target'):
                        target = child.getAttribute('target')
                        if target == '': target = '_blank'
                        result.append(
                            ' target="%s"' % mangle.entities(target))
                    result.append('>')
                    result.append(self._renderHtmlHelper(child, view_type))
                    result.append('</a>')
            elif child.nodeName == 'underline':
                result.append('<u>')
                result.append(self._renderHtmlHelper(child, view_type))
                result.append('</u>')
            elif child.nodeName == 'index':
                index = mangle.entities(child.getAttribute('name'))
                result.append(
                    '<a class="index-element" name="%s">' % index)
            
                # in nested lists and tables we cannot rely on the
                # correct view_type passed in, so check for edit_mode
                # from the request
                edit_mode = node.REQUEST.get('edit_mode', 0)
                if edit_mode:
                    result.append('[[%s]]' % index)
                result.append('</a>')
            elif child.nodeName == 'br':
                result.append('<br />')
            else:
                result.append('<span class="error">%s</span> ' % 
                    self._renderHtmlHelper(child, view_type))
        return ''.join(result)


InitializeClass(GlossaryParagraphSupport)


def renderGlossaryLink(paragraphsupport, gloss_entry, context, view_type):
    """Renders the glossary link

    Uses the default or customized renderer specified in the Silva
    Glossary Service.
    """
    # Get the glossary entry
    context_container = context.get_container()

    # Generate an unambiguous XML id
    phys_path = gloss_entry.getPhysicalPath()
    gloss_id = '.'.join(phys_path[1:])

    # Add the glossary link to a dictionary. This will be processed
    # at the end of the page.
    request = context.REQUEST
    if not request.has_key('silva_glossary'):
        # Create new dictionary if no glossary link has been found yet.
        request.set('silva_glossary',{gloss_id: gloss_entry})
    else:
        request['silva_glossary'][gloss_id] = gloss_entry

    # Render the link's content
    link_content = paragraphsupport._renderHtmlHelper(context, view_type)

    # Render the link by the default or customized renderer.
    service_silva_glossary = context_container.service_silva_glossary
    rendered_link = service_silva_glossary.render_link(
        gloss_id, gloss_entry, link_content)

    # Remove a trailing line break assuming that the glossary link should
    # be a inline element. Page templates cannot render a string without
    # a trailing line break.
    if rendered_link[-1] == '\n':
        rendered_link = rendered_link[:-1]
    return rendered_link

